@@ -3,9 +3,9 @@
 #' Read texts and create a \code{keyATM_docs} object, which is a list of texts.
 #'
 #' 
-#' @param texts input. keyATM takes dfm, data.frame, \pkg{tibble} tbl_df, or a vector of file paths.
-#' @param encoding character. Only used when \code{texts} is a vector of file paths. Default is "UTF-8".
-#' @param check logical. If \code{TRUE}, check whether there is nothing wrong with the structure of texts. Default is \code{TRUE}.
+#' @param texts input. keyATM takes quanteda dfm (dgCMatrix), data.frame, \pkg{tibble} tbl_df, or a vector of file paths.
+#' @param encoding character. Only used when \code{texts} is a vector of file paths. Default is \code{UTF-8}.
+#' @param check logical. If \code{TRUE}, check whether there is anything wrong with the structure of texts. Default is \code{TRUE}.
 #'
 #' @return a list whose elements are splitted texts. The length of the list equals to the number of documents.
 #'
@@ -23,7 +23,6 @@
 #'  keyATM_docs <- keyATM_read(texts = files) 
 #' 
 #' }
-#'
 #' @import magrittr
 #' @importFrom rlang .data
 #' @export
@@ -41,7 +40,7 @@ keyATM_read <- function(texts, encoding = "UTF-8", check = TRUE)
     text_dfm <- NULL
     files <- NULL
     text_df <- tibble::as_tibble(texts)
-  } else if (class(texts) == "dfm") {
+  } else if (class(texts) == "dfm" | "dgCMatrix" %in% class(texts)) {
     message("Using quanteda dfm.")
     text_dfm <- texts
     files <- NULL
@@ -85,17 +84,9 @@ keyATM_read <- function(texts, encoding = "UTF-8", check = TRUE)
   # check whether there is nothing wrong with the structure of texts
   if (check) {
     check_vocabulary(unique(unlist(W_raw, use.names = FALSE, recursive = FALSE))) 
+    doc_index <- get_doc_index(W_raw, check = TRUE)
   }
 
-  # Check document length
-  lapply(W_raw, length) %>% unlist(use.names = FALSE) -> len
-  len_zero_index <- (1:length(W_raw))[(1:length(W_raw))[len == 0]]
-  if (length(len_zero_index) != 0) {
-    stop("Number of documents with 0 length: ", length(len_zero_index), "\n",
-         "Please review the preprocessing steps.") 
-  }
-
-  
   # Assign class
   class(W_raw) <- c("keyATM_docs", class(W_raw))
 
@@ -139,44 +130,33 @@ summary.keyATM_docs <- function(object, ...)
 #'
 #' Visualize the proportion of keywords in the documents.
 #'
-#' 
 #' @param docs a keyATM_docs object, generated by \code{keyATM_read()} function
 #' @param keywords a list of keywords
 #' @param prune logical. If \code{TRUE}, prune keywords that do not appear in `docs`. Default is \code{TRUE}.
-#' @param label_size the size of keyword labels in the output plot. Default is 3.2.
-#'
-#' @return A list containing \describe{
-#'    \item{figure}{a ggplot2 object}
-#'    \item{values}{a tibble object that stores values}
-#'    \item{keywords}{a list of keywords that appear in documents}
-#' }
-#'
+#' @param label_size the size of keyword labels in the output plot. Default is \code{3.2}.
+#' @return keyATM_fig object
 #' @examples
 #' \dontrun{
 #'  # Prepare a keyATM_docs object
 #'  keyATM_docs <- keyATM_read(input)
 #'   
 #'  # Keywords are in a list  
-#'  keywords <- list(
-#'                    c("education", "child", "student"),  # Education
-#'                    c("public", "health", "program"),  # Health
-#'                  )
+#'  keywords <- list(Education = c("education", "child", "student"),
+#'                   Health    = c("public", "health", "program"))
 #'
 #'  # Visualize keywords
 #'  keyATM_viz <- visualize_keywords(keyATM_docs, keywords)
 #'
 #'  # View a figure
 #'  keyATM_viz
-#'    # Or: `keyATM_viz$figure`
 #' 
 #'  # Save a figure 
 #'  save_fig(keyATM_viz, filename)
-#'
 #' }
-#'
 #' @import magrittr
 #' @import ggplot2
 #' @importFrom rlang .data
+#' @seealso [save_fig()]
 #' @export
 visualize_keywords <- function(docs, keywords, prune = TRUE, label_size = 3.2)
 {
@@ -247,10 +227,9 @@ visualize_keywords <- function(docs, keywords, prune = TRUE, label_size = 3.2)
       theme_bw()
 
   keyATM_viz <- list(figure = visualize_keywords, values = temp, keywords = keywords)
-  class(keyATM_viz) <- c("keyATM_viz", class(keyATM_viz))
+  class(keyATM_viz) <- c("keyATM_fig", "keyATM_viz", class(keyATM_viz))
   
   return(keyATM_viz)
-
 }
 
 
@@ -302,46 +281,27 @@ check_keywords <- function(unique_words, keywords, prune)
   return(keywords)
 }
 
-#' Print a keyword plot
-#' 
-#' @noRd
-#' @export
-print.keyATM_viz <- function(x, ...)
+
+get_doc_index <- function(docs, check = FALSE)
 {
-  print(x$figure)  
-}
-
-#' Check summary statistics of the keywords
-#' 
-#' @noRd
-#' @export
-summary.keyATM_viz <- function(object, ...)
-{
-  return(object$values)  
-}
-
-
-#' Save a keyATM_viz object
-#'
-#' @param x a keyATM_viz object (see \code{visualize_keywords()})
-#' @param file a character
-#'
-#' @export
-save.keyATM_viz <- function(x, file = stop("'file' must be specified"))
-{
-  saveRDS(x, file = file)
-}
-
-
-#' Save a keyword plot
-#'
-#' @param x a keyATM_viz object (see \code{visualize_keywords()})
-#' @param file a character
-#'
-#' @export
-save_fig.keyATM_viz <- function(x, file = stop("'file' must be specified"))
-{
-  ggplot2::ggsave(x$figure, file = file)
+  lapply(docs, length) %>% unlist(use.names = FALSE) -> len
+  index <- 1:length(docs)
+  nonzero_index <- index[index[len != 0]]
+  zero_index <- index[index[len == 0]]
+  if (length(zero_index) != 0) {
+    if (check) {
+      warning("Number of documents with 0 length: ", length(zero_index), "\n",
+              "This may cause invalid covariates or time index.", "\n",
+              "Please review the preprocessing steps.", "\n",
+              "Document index to check: ", paste(zero_index, collapse = ", "),
+              immediate. = TRUE)     
+    } else {
+      warning("Number of documents dropped because of 0 length: ", length(zero_index), "\n",
+              "Document index to check: ", paste(zero_index, collapse = ", "),
+              immediate. = TRUE) 
+    }
+  }
+  return(nonzero_index)
 }
 
 
@@ -375,10 +335,12 @@ keyATM_fit <- function(docs, model, no_keyword_topics,
   keywords <- check_arg(keywords, "keywords", model, info)
 
   # Get Info
+  info$use_doc_index <- get_doc_index(docs)
+  docs <- docs[info$use_doc_index]
   info$num_doc <- length(docs)
   info$keyword_k <- length(keywords)
   info$total_k <- length(keywords) + no_keyword_topics
-  info$num_core <- parallel::detectCores(all.tests = FALSE, logical = TRUE) - 2L
+  info$num_core <- max(1, parallel::detectCores(all.tests = FALSE, logical = TRUE) - 2L)
 
   # Set default values
   model_settings <- check_arg(model_settings, "model_settings", model, info)
@@ -386,6 +348,7 @@ keyATM_fit <- function(docs, model, no_keyword_topics,
   options <- check_arg(options, "options", model, info)
   info$parallel_init <- options$parallel_init
 
+
   ##
   ## Initialization
   ##
@@ -425,7 +388,8 @@ keyATM_fit <- function(docs, model, no_keyword_topics,
   rm(res)
 
   # Organize
-  stored_values <- list()
+  stored_values <- list(vocab_weights = rep(-1, length(info$wd_names)),
+                        doc_index = info$use_doc_index)
 
   if (model %in% c("base", "lda", "label")) {
     if (options$estimate_alpha)
@@ -484,6 +448,12 @@ keyATM_fit <- function(docs, model, no_keyword_topics,
   ##
   ## Fitting
   ##
+  return(fitting_models(key_model, model, options))
+}
+
+
+fitting_models <- function(key_model, model, options)
+{
   message(paste0("Fitting the model. ", options$iterations, " iterations..."))
   set.seed(options$seed)
 
@@ -542,6 +512,10 @@ check_arg <- function(obj, name, model, info = list())
   if (name == "options") {
     return(check_arg_options(obj, model, info))  
   }
+
+  if (name == "vb_options") {
+    return(check_arg_vboptions(obj, model, info))  
+  }
 }
 
 
@@ -623,6 +597,8 @@ check_arg_model_settings <- function(obj, model, info)
       stop("Please provide `obj$covariates_data`.")  
     }
 
+    obj$covariates_data <- as.data.frame(obj$covariates_data)[info$use_doc_index, , drop = FALSE]
+
     if (nrow(obj$covariates_data) != info$num_doc) {
       stop("The row of `model_settings$covariates_data` should be the same as the number of documents.")  
     }
@@ -634,7 +610,7 @@ check_arg_model_settings <- function(obj, model, info)
     if (is.null(obj$covariates_formula)) {
       warning("`covariates_formula` is not provided. keyATM uses the matrix as it is.", immediate. = TRUE)
       obj$covariates_formula <- NULL  # do not need to change the matrix
-      obj$covariates_data_use <- as.matrix(obj$covariates_data) 
+      obj$covariates_data_use <- obj$covariates_data
     } else if (is.formula(obj$covariates_formula)) {
       message("Convert covariates data using `model_settings$covariates_formula`.")
       obj$covariates_data_use <- stats::model.matrix(obj$covariates_formula,
@@ -722,6 +698,8 @@ check_arg_model_settings <- function(obj, model, info)
       stop("`model_settings$time_index` is not provided.")
     }
 
+    obj$time_index <- obj$time_index[info$use_doc_index]
+
     if (length(obj$time_index) != info$num_doc) {
       stop("The length of the `model_settings$time_index` does not match with the number of documents.")  
     }
@@ -733,6 +711,7 @@ check_arg_model_settings <- function(obj, model, info)
     if (max(obj$time_index) < obj$num_states)
       stop("`model_settings$num_states` should not exceed the maximum of `model_settings$time_index`.")
 
+
     check <- unique(obj$time_index[2:length(obj$time_index)] - stats::lag(obj$time_index)[2:length(obj$time_index)])
     if (sum(!unique(check) %in% c(0,1)) != 0)
       stop("`model_settings$num_states` does not increment by 1.")
@@ -748,6 +727,9 @@ check_arg_model_settings <- function(obj, model, info)
     if (is.null(obj$labels)) {
       stop("`model_settings$labels` is not provided.")
     }
+
+    obj$labels <- obj$labels[info$use_doc_index]
+
     if (length(obj$labels) != info$num_doc) {
       stop("The length of `model_settings$labels` does not match with the number of documents")
     }
@@ -767,7 +749,6 @@ check_arg_model_settings <- function(obj, model, info)
   }
 
   show_unused_arguments(obj, "`model_settings`", allowed_arguments)
-
   return(obj)
 }
 
@@ -829,7 +810,6 @@ check_arg_priors <- function(obj, model, info)
   }
 
   show_unused_arguments(obj, "`priors`", allowed_arguments)
-
   return(obj)
 }
 
